% This program was used in publication "Ndc80 complex, a conserved coupler for kinetochore-microtubule motility, 
% is a sliding molecular clutch" by V. Demidov et al. 2025.
% This program builds, plots, analyzes, and saves velocity histograms. Then
% it plots and saves relative distance between dumbbell beads for further
% examination of its stability.
% Input: folder, QPD1 and QPD2 data files, each containing two columns: trapped bead position in nm and trap position in nm. 

%% Initialize parameters
down =0; % direction of plus-end gliding. 1 - down, 0 - up
Nstart = 1; % analyze starting from this ped#
Nped = 1; % N of pedestals in folder to analyze
peak_width_coeff = 0.8; % default = 0.8
dt = 15*1e-6; %time frame

%% Cycle through folders with data

ini_path=pwd; % folder with all ped# folders


for ii=1:1:2
    if ii==1
        force = ['1.0pN'; '2.0pN'; '3.0pN'; '4.0pN';'5.0pN'; '6.0pN';'7.0pN';'8.0pN']; % 
    elseif ii==2
        force = ['10.0pN';'12.0pN';'14.0pN';'16.0pN';'18.0pN';'20.0pN'];%


    end
    
    
force = char(force);
pattern = ['*__0_61*']; % pattern to analyze 


%% create lists all files to analyze
for l=Nstart:1:Nped+Nstart-1
ped_i = char(['ped',num2str(l)]) % create char variable "ped#"
for v=1:size(force,1)
path_f=fullfile(ini_path,'\',ped_i,'\',force(v,:)); % create char path "path_f" to folder with data
if exist(path_f)==7 % check if path exists
    force(v,:)
    cd(path_f)
listing =dir(pattern); % lists all files to analyze


close all
clear A
clear data
%% confirm that files are loaded in right order
if listing(1).name(3)~=char('1')
    error('wrong file loaded')
elseif listing(2).name(3)~='2'
    error('wrong file loaded')
end

%% load files
QD1=load(listing(1).name);
QD2=load(listing(2).name);

if size(QD1,1)>2002500
    error('wrong file size')
end

%% pick right columns for analysis
if down == 1
data(:,1) = QD1(:,1);
elseif down==-1
data(:,1) = (QD1(:,1)+QD2(:,1))/2;
else
data(:,1) = QD2(:,1);
end

data(:,2) = (QD1(:,2)+QD2(:,2))./2;

name = data;
sz=size(name);
poscol = 1; 
A(:,1) = name(:,poscol);
A(:,2) = (QD1(:,2)-QD1(:,1)+QD2(:,2)-QD2(:,1))./2;
clear name

%% parameters for instantanious velocity filtering
k=1;
clear UpIndex index;
clear DownIndex;
W0fast=8; %smoothing window 1
W0slow=40; %smoothing window 2
W = 1;   %raw signal smoothing window
sweepthr = 0.000; % only sweeps longer then sweepthr will be analysed
time=0:dt:(sz(1,1)-1)*dt;




%% Find data frames when applied force switches direction and separate periods of up- and down-directed motion
j=1;
Force_multiplicated = A(1:end-1,2).*A(2:end,2);
index = find(Force_multiplicated<0);
%hotfix: takes out first sweep if it is down direction
if sign(A(1,2))<0
A(1:index,:)=[];
index = index-index(1);
index(1)=[];
end
sz=size (A);


if sign(A(1,2))>0
    UpIndex(1,1)=1;
    ini=1;
else ini=-1;
    DownIndex(1,1)=1;
end

k=1;
for i=1:1:size(index)/2
    if ini==1
        UpIndex(k,2)=index(2*i-1);
        UpIndex(k+1,1)=index(2*i)+1;
        k=k+1;
    else
        UpIndex(k,1)=index(2*i-1)+1;
        UpIndex(k,2)=index(2*i);
        k=k+1;
    end
end

k=1;

for i=1:1:floor(size(index)/2)
        DownIndex(k,1)=index(2*i-1)+1;
        DownIndex(k,2)=index(2*i);
        k=k+1;
end

if UpIndex(end,1)>DownIndex(end,1)
    UpIndex(end,2) = size(A,1);
else
    DownIndex(end,2) = size(A,1);
end
k=1;


%% Concatenate Up & Down motion into two separate time series data sets
clear Up Down;
Ashift = A(:,1);
for i=1:1:max(size(UpIndex))-1
    shiftUp = Ashift(UpIndex(i,2),1)-Ashift(UpIndex(i+1,1),1);
    regionUp = UpIndex(i+1,1):UpIndex(i+1,2);
    Ashift(regionUp,1) = Ashift(regionUp,1)+shiftUp;
    
end  
for i=1:1:max(size(DownIndex))-1
    regionDown = DownIndex(i+1,1):DownIndex(i+1,2);
    shiftDown = Ashift(DownIndex(i,2),1)-Ashift(DownIndex(i+1,1),1);
    Ashift(regionDown,1) = Ashift(regionDown,1)+shiftDown;
end

Up = zeros(size(A,1),1);

lastframe = 0;
for i=1:1:max(size(UpIndex))
    regionUp = UpIndex(i,1):UpIndex(i,2);
    framelength = UpIndex(i,2)-UpIndex(i,1)+1;
    Up (lastframe+1:lastframe+framelength) = Ashift(regionUp,1);
    lastframe = lastframe+framelength;
end
    Up(lastframe+1:end) = [];

Down = zeros(size(A,1),1);

lastframe = 0;
for i=1:1:max(size(DownIndex))
    regionDown = DownIndex(i,1):DownIndex(i,2);
    framelength = DownIndex(i,2)-DownIndex(i,1)+1;
    Down (lastframe+1:lastframe+framelength) = Ashift(regionDown,1);
    lastframe = lastframe+framelength;
end
    Down(lastframe+1:end) = [];

    
%% For each direction calculate first derivative
clear derUp derUpsm derDown derDownsm

for i=1:1:size(Up)-1
    derUp(i)=(Up(i+1)-Up(i))/dt;
end

for i=1:1:size(Down)-1
    derDown(i)=(Down(i+1)-Down(i))/dt;
end

%% Apply gaussian filters to the first derivative
derUpsmfast=smoothts(derUp,'g',8*W0fast,W0fast^2);
derUpsmslow=smoothts(derUp,'g',8*W0slow,W0slow^2);
derUpsmslow=derUpsmslow./1000;

derDownslow = -smoothts(derDown,'g',8*W0slow,W0slow^2);
derDownslow=derDownslow./1000;
derDownfast = -smoothts(derDown,'g',8*W0fast,W0fast^2);
derDownfast = derDownfast./1e3;



%% plot results
figure(1)
title ('Bead 1 position')
Adec=A(1:2:end,1);

t = 0:1:length(Adec)-1;
t=t'*dt*2;
plot(t, Adec(:,1),'.','MarkerSize',0.1); set(gca,'FontSize',14);
axis ([0 7.1 -250 250]) % ([0 7.1 -155 155])
xlabel('time, s', 'FontSize', 14); ylabel('position, nm','FontSize', 14);


figure(2)
binsz = 2;
hist(derUpsmslow,min(derUpsmslow):binsz:max(derUpsmslow)); set(gca,'FontSize',14);
hold all
[histy,histx] = hist(derUpsmslow,min(derUpsmslow):binsz:max(derUpsmslow));
[pks,locs,w,p]=findpeaks(histy,'MinPeakWidth',20); % small peaks usually have width ~ 2-3 points, right peaks have ~20-30 ponts
[peak_max_ind,ind_peak] = max(locs);
peak_width = w(ind_peak);


plot(histx(locs), w,'.r')

%% detect peaks on each histogram, fit them with Gaussian, and plot results
if peak_max_ind-peak_width<1
    peak_start = 1;
else
    peak_start = peak_max_ind-peak_width;
end

if peak_max_ind+peak_width>length(histx)
    peak_width = peak_width_coeff*w(ind_peak);
    display('peak width changed')
end

histx = histx(peak_start:peak_max_ind+peak_width); % 20 poits - approximate width of free vel peak
if length(histx)>3
histy = histy(peak_start:peak_max_ind+peak_width); % 20 poits - approximate width of free vel peak
fitting2 = fit (histx', histy', 'gauss1'); % use [a,b,c]=coeffvalues(fitting1) to get fitted values
coeff = coeffvalues(fitting2);
plot(fitting2)
% plot(thr, 0:1e3, '--r')
title(['up; insta vel distribution; mean = ', num2str(coeff(2),3),' +- ',num2str(coeff(3),3),'um/s, '],'FontSize', 14);
end
xlabel('velocity, um/s', 'FontSize', 14); ylabel('counts','FontSize', 14);
xlim([-0.5e1 ii*3.5e2])
axis2=axis;
    set(gcf, 'OuterPosition',[547.4 241 ii*574.4 508]);
hold off

figure(3)
binsz = 2;
hist(derDownslow,min(derDownslow):binsz:max(derDownslow)); set(gca,'FontSize',14);  %   derUpsmfast
hold on
[histy,histx] = hist(derDownslow,min(derDownslow):binsz:max(derDownslow));
[pks,locs,w,p]=findpeaks(histy,'MinPeakWidth',20); % small peaks usually have width ~ 2-3 points, right peaks have ~20-30 ponts
    
[peak_max_ind,ind_peak] = max(locs);
peak_width = w(ind_peak);


plot(histx(locs), w,'.r')

if peak_max_ind-peak_width<1
    peak_start = 1;
else
    peak_start = peak_max_ind-peak_width;
end

if peak_max_ind+peak_width>length(histx)
    peak_width = peak_width_coeff*w(ind_peak);
    display('peak width changed')
end

histx = histx(peak_start:peak_max_ind+peak_width); % 20 poits - approximate width of free vel peak
if length(histx)>3
histy = histy(peak_start:peak_max_ind+peak_width); % 20 poits - approximate width of free vel peak
fitting2 = fit (histx', histy', 'gauss1'); % use [a,b,c]=coeffvalues(fitting1) to get fitted values
coeff = coeffvalues(fitting2);
plot(fitting2);
% plot(thr, 0:1e3, '--r')
title(['down; insta vel distribution; mean = ', num2str(coeff(2),3),' +- ',num2str(coeff(3),3),'um/s, '],'FontSize', 14);
end
xlabel('velocity, um/s', 'FontSize', 14); ylabel('counts','FontSize', 14);
xlim([-0.5e1 ii*3.5e2])
axis3=axis;
    set(gcf, 'OuterPosition',[547.4 241 ii*574.4 508]);
hold off

figure(4)

%% Plot relative distance between dumbbell beads
Adec1=QD1(1:10:end,1);
Adec2=QD2(1:10:end,1);
t = 0:1:length(Adec1)-1;
t=t'*dt*10;
Diff_mean = mean(Adec1(:,1)-Adec2(:,1));
Diff_SD = std(Adec1(:,1)-Adec2(:,1));

plot(t, Adec1(:,1)-Adec2(:,1)); set(gca,'FontSize',14);
title(['QD1-QD2; mean = ', num2str(Diff_mean,3),' +- ',num2str(Diff_SD,3),'nm'],'FontSize', 14);
axis([0 31 -155 155])
xlabel('time, s', 'FontSize', 14); ylabel('position, nm','FontSize', 14);


%%  axes parameters
y2=7e4;  %y2 = max(axis2(4),axis3(4));
figure(2)
ylim([0 y2])
figure(3)
ylim([0 y2])

%% save graphs
    saveas(figure(2),'velocity distr UP')
    saveas(figure(2),'velocity distr UP.bmp')
    saveas(figure(1),'bead position, 7sec')
    saveas(figure(3),'velocity distr DOWN.bmp')
    saveas(figure(3),'velocity distr DOWN')
    saveas(figure(4),'QD1_QD2_subtr.bmp')
    saveas(figure(4),'QD1_QD2_subtr')

end
end
end
end