% This program was used in publication "Ndc80 complex, a conserved coupler for kinetochore-microtubule motility, 
% is a sliding molecular clutch" by V. Demidov et al. 2025.
% This program collects all events detected in the recording into one array for further analysis
% Input:
% 1. folder with QPD1 and QPD2 data files, each containing two columns: trapped bead position in nm and trap position in nm. 
% 2. This folder should be fed to the Automatic_event_detection.m program
% in advance in order to generate files "shifteventssweep.txt" and "UpIndex.txt"

clear all
close all , warning off; 

down = 1;
ped_to_analyze=[9]; % choose pedestal to analyze

dt=15e-6;

pattern = ['*__0_61*']; % pattern to analyze 
force = {'4.0pN'; '8.0pN';'12.0pN'; '16.0pN'; '20.0pN'};
ini_path = pwd;

for l=ped_to_analyze
ped_i = char(['ped',num2str(l)]) % create char variable "ped#"
for v=1:size(force,1)
path_f=fullfile(ini_path,ped_i,char(force(v,:))); % create char path "path_f" to folder with force data
if exist(path_f)==7 % check if path exists
    force(v,:)
    cd(path_f)
listing =dir(pattern);
if listing(1).name(3)~=char('1')
    error('wrong file loaded')
elseif listing(2).name(3)~='2'
    error('wrong file loaded')
end

%% load files
QD1=load(listing(1).name);
QD2=load(listing(2).name);

if down==0
    if exist(fullfile(path_f,'up')) == 7
        new_folder = fullfile(path_f,'up');  
    else
        new_folder = fullfile(path_f,'up_no_beaks'); 
    end


elseif down==1
    if exist(fullfile(path_f,'down')) == 7
        new_folder = fullfile(path_f,'down'); 
    else
        new_folder = fullfile(path_f,'down_no_beaks'); 
    end
end

cd (new_folder)

if exist(' shift_events_sweep.txt')==2
    shift_events_sweep = load(' shift_events_sweep.txt');
else
    load('shift_events_sweep.txt')
end

if exist(' eventindreal.txt')==2
    eventindreal=load(' eventindreal.txt');
else
    load('eventindreal.txt')
end

if exist(' UpIndex.txt')==2
    UpIndex = load(' UpIndex.txt');
else
    load('UpIndex.txt')
end


if exist(' bt_slope_std_filt.txt')==2
    bt_slope_std_filt = load (' bt_slope_std_filt.txt');
else
    load ('bt_slope_std_filt.txt')
end
    
durthr = min(bt_slope_std_filt(:,1));

shifteventssweep = shift_events_sweep;
cd (listing(1).folder)


%% Create array of force direction switch indexes to take out first sweep if it is in DOWN direction so that program functions correctly

if down == 1
data(:,1) = QD1(:,1);
else
data(:,1) = QD2(:,1);
end

close all
clear A
sz=size(data);
if sz(2)==1
    poscol = 1;
else
    poscol = 2;
end

A(:,1) = data(:,poscol);
A(:,2) = (QD1(:,2)-QD1(:,1)+QD2(:,2)-QD2(:,1))./2;

j=1;
flag = 0;
i=1;
while flag==0

if (sign(A(i,2)*A(i+1,2))==-1)
    index(j,1)=i;
    j=j+1;
    flag = 1;
end
i = i+1;

end
UpIndex = UpIndex+index(1);

QD=A;clear A;

%% Signal preparation: 1. remove interactions shorter than 2 ms
todel = [];
for i=1:1:size(eventindreal,1)
    if dt*(eventindreal(i,2)-eventindreal(i,1))<durthr
        todel(end+1)= i;
    end
end

if isempty(todel)
    else
        eventindreal(todel,:) = [];
        shifteventssweep(todel,:)=[];
end


%% filtering sweeps accounted twice
temp=[];
temp = shifteventssweep(2:end,3)-shifteventssweep(1:end-1,3);
to_del = find(temp == 0);
n = length(to_del);
disp(['number of sweeps accounted twice:', num2str(n),'/', num2str(n+size(shifteventssweep,1))]);
shifteventssweep(to_del,:)=[];

%% 3. Include only gliding events that last 5 or more interacting sweeps

ind_to_incl = [];
f_l_ind=[];
temp=[];
temp=shifteventssweep(2:end,3)-shifteventssweep(1:end-1,3);

if temp(1)<=1
f_l_ind(1,1)=1;
else
f_l_ind(1,:)=[1,1];
end

for i=2:1:length(temp)
    if temp(i)<=1&&temp(i-1)>1
        f_l_ind(end+1,1) = i;
    
    elseif temp(i)>1&&temp(i-1)<=1
        f_l_ind(end,2) = i;
    elseif temp(i)>1&&temp(i-1)>1
        f_l_ind(end+1,:) = [i,i];
    end
end    

if temp(end)<=1
f_l_ind(end,2)=length(temp)+1;
else
f_l_ind(end+1,:)=[length(temp)+1,length(temp)+1];
end

%% generating indexes of frames with plus end interacting sweeps
for i=1:size(f_l_ind,1)
    if f_l_ind(i,2)-f_l_ind(i,1)+1>=5&&f_l_ind(i,2)-f_l_ind(i,1)+1<=999999
        ind_to_incl=[ind_to_incl,f_l_ind(i,1):1:f_l_ind(i,2)];
    end
end

%% generating indexes of frames with minus end interacting sweeps
ind_to_incl_minus=[];
for i=1:size(f_l_ind,1)
    if f_l_ind(i,2)-f_l_ind(i,1)+1>=5&&f_l_ind(i,2)-f_l_ind(i,1)+1<=999999
        ind_to_incl_minus=[ind_to_incl_minus,f_l_ind(i,1):1:f_l_ind(i,2)-1];
    end
end

%% plot the signal
time_glob = [1 : 1 : size(QD,1)]'*dt;


%% Collecting of events
data = [];
sweep_ind_save=[];
shift = 0;
edge = [];
data_no_shift =[];
out=[];
resid_out = [];
y_lad = 0;
y=[];
y_minus=[];
min_sz = 1/dt; % starting mix_sz valure is 1 s

s_i = [shifteventssweep(ind_to_incl,3)];
sweep_ind = [UpIndex(s_i,1),UpIndex(s_i,2)];
ind_to_incl_upd = ind_to_incl;

clear y
N = length(ind_to_incl_upd);

plot(QD)
hold all
    
for i=1:N
    s_i = [shifteventssweep(ind_to_incl_upd(i),3)]; 
    sweep_ind = [UpIndex(s_i,1),UpIndex(s_i,2)];
    plot([UpIndex(s_i,1):1:UpIndex(s_i,2)], QD([UpIndex(s_i,1):1:UpIndex(s_i,2)]), 'black')
    sz = sweep_ind(2)-sweep_ind(1)+1;
    min_sz = min(min_sz,sz(1));
    y(1:sz,i) = [QD(sweep_ind(1):sweep_ind(2))'-QD(sweep_ind(1))]*(-2*down+1);
end

clear DownIndex y_minus

DownIndex(:,1) = UpIndex(1:end-1,2)+1;
DownIndex(:,2) = UpIndex(2:end,1)-1;

N = length(ind_to_incl_minus);
for i=1:N
    s_i = [shifteventssweep(ind_to_incl_minus(i),3)]; 
    sweep_ind = [DownIndex(s_i,1),DownIndex(s_i,2)];
%     plot([DownIndex(s_i,1):1:DownIndex(s_i,2)], QD([DownIndex(s_i,1):1:DownIndex(s_i,2)]), 'red')

    sz = sweep_ind(2)-sweep_ind(1)+1;
    min_sz = min(min_sz,sz(1));
    y_minus(1:sz,i) = [QD(sweep_ind(1):sweep_ind(2))'-QD(sweep_ind(1))]*(2*down-1);
end


%% save interacting sweeps
figure
plot(y)
figure
plot(y_minus)

save_folder = fullfile(new_folder,'interacting_sweeps');
mkdir(save_folder)
save([save_folder,'\sweeps_plus_end.txt'], 'y', '-ascii', '-double', '-tabs')
save([save_folder,'\sweeps_minus_end.txt'], 'y_minus', '-ascii', '-double', '-tabs')
end
end
end